local nkUIInfo, nkUI = ...

function nkUI.nkSlider(name, parent) 

	local elementColor = {1, 1, 1, 1}

	local slider = UI.CreateFrame ('Frame', name, parent)
	local sliderLane = UI.CreateFrame ('Frame', name .. 'lane', slider)
	local sliderLaneInner = UI.CreateFrame ('Frame', name .. 'inner', sliderLane)
	local sliderPos = UI.CreateFrame ('Frame', name .. 'pos', sliderLaneInner)

	local properties = {}

	function slider:SetValue(property, value)
		properties[property] = value
	end
	
	function slider:GetValue(property)
		return properties[property]
	end
	
	slider:SetValue("name", name)
	slider:SetValue("parent", parent)
	
	slider:SetWidth(100)
	slider:SetHeight(14)
	
	sliderLane:SetPoint("TOPLEFT", slider, "TOPLEFT", 0, 2)
	sliderLane:SetPoint("BOTTOMRIGHT", slider, "BOTTOMRIGHT", 0, -2)
	sliderLane:SetBackgroundColor(elementColor[1], elementColor[2], elementColor[3], elementColor[4])
	sliderLane:SetWidth(100)
	sliderLane:SetHeight(10)
	
	sliderLaneInner:SetPoint("TOPLEFT", sliderLane, "TOPLEFT", 1, 1)
	sliderLaneInner:SetPoint("BOTTOMRIGHT", sliderLane, "BOTTOMRIGHT", -1, -1)
	sliderLaneInner:SetBackgroundColor(0, 0, 0, 1)
	
	sliderPos:SetPoint("LEFTCENTER", sliderLane, "LEFTCENTER")
	sliderPos:SetBackgroundColor(elementColor[1], elementColor[2], elementColor[3], elementColor[4])
	sliderPos:SetWidth(14)
	sliderPos:SetHeight(14)
	
	local mouseDown = false
	
	sliderPos:EventAttach(Event.UI.Input.Mouse.Left.Down, function ()
		mouseDown = true
	end, name .. "pos_Left_Down")
	
	sliderPos:EventAttach(Event.UI.Input.Mouse.Left.Up, function ()
		mouseDown = false
	end, name .. "pos_Left_Up")
	
	sliderPos:EventAttach(Event.UI.Input.Mouse.Left.Upoutside , function ()
		mouseDown = false
	end, name .. "pos_Left_Upoutside")
	
	sliderPos:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function ()
		if mouseDown then slider:ProcessMove() end
	end, name .. "pos_Left_Up")
	
	function slider:ProcessMove()
		
		if Inspect.Mouse().y < (sliderLane:GetTop() - 14) or Inspect.Mouse().y > (sliderLane:GetTop() + sliderLane:GetHeight() + 14) then mouseDown = false end
		if Inspect.Mouse().x < (sliderLane:GetLeft() - 40) or Inspect.Mouse().x > (sliderLane:GetLeft() + sliderLane:GetWidth() + 40) then mouseDown = false end
		
		local range = self:GetValue("range")
		
		if range == nil then mouseDown = false end
		
		if mouseDown == false then return end

		local x = Inspect.Mouse().x
		if x < sliderLane:GetLeft() then x = sliderLane:GetLeft() end
		if x > sliderLane:GetLeft() + sliderLane:GetWidth() - 14 then x = sliderLane:GetLeft() + sliderLane:GetWidth() - 14 end
		
		local curdivX = x - sliderLane:GetLeft()
		
		local valuePerPixel = (range[2] - range[1]) / sliderLane:GetWidth()
		local newValue = curdivX * valuePerPixel + range[1]
		
		if newValue < range[1] then newValue = range[1] end
		if newValue > range[2] then newValue = range[2] end
		
		self:SetValue("value", newValue)

		sliderPos:SetPoint ("CENTERLEFT", sliderLane, "CENTERLEFT", curdivX, 0)
		
		Library.nkUI.handler[name]["SliderChanged"](newValue)
	end
	
	function slider:SetRange(from, to)
		self:SetValue("range", { from, to })
	end
	
	function slider:AdjustValue(newValue)
		local range = self:GetValue("range")
		if range == nil then return end
		
		self:SetValue("value", newValue)
		
		local newX = math.floor((sliderLane:GetWidth() -14) / (range[2] - range[1] + 1) * (newValue-1))
		sliderPos:SetPoint ("CENTERLEFT", sliderLane, "CENTERLEFT", newX, 0)
		
		Library.nkUI.handler[name]["SliderChanged"](newValue)
		
	end
	
	function slider:SetColor (r, g, b, a)
		elementColor = { r, g, b, a }
	
		sliderLane:SetBackgroundColor(r, g, b, a)
		sliderPos:SetBackgroundColor(r, g, b, a)
	end
			
	Library.nkUI.handler[name] = {}
	Library.nkUI.Event[name] = {}
	Library.nkUI.handler[name]["SliderChanged"], Library.nkUI.Event[name]["SliderChanged"] = Utility.Event.Create(nkUIInfo.identifier, name .. "SliderChanged")
	
	return slider
	
end